@@ -211,19 +211,20 @@ static void sbox_uint64_lowmc_255_255_4(mzd_local_t* in) {
 #if defined(WITH_OPT)
 #if defined(WITH_SSE2) || defined(WITH_NEON)
 ATTR_TARGET_S128
-static inline void sbox_s128_full(mzd_local_t* in, const word128* mask_a, const word128* mask_b,
-                                  const word128* mask_c) {
-  word128 x0m[2] ATTR_ALIGNED(alignof(word128)), x1m[2] ATTR_ALIGNED(alignof(word128)),
-      x2m[2] ATTR_ALIGNED(alignof(word128));
-  mm128_and_256(x0m, CONST_BLOCK(in, 0)->w128, mask_a);
-  mm128_and_256(x1m, CONST_BLOCK(in, 0)->w128, mask_b);
-  mm128_and_256(x2m, CONST_BLOCK(in, 0)->w128, mask_c);
+static inline void sbox_s128_full(mzd_local_t* in, const mzd_local_t* mask_a,
+                                  const mzd_local_t* mask_b, const mzd_local_t* mask_c) {
+  word128 inm[2] = {mm128_load(&in->w64[0]), mm128_load(&in->w64[2])};
+  word128 x0m[2] = {mm128_load(&mask_a->w64[0]), mm128_load(&mask_a->w64[2])};
+  word128 x1m[2] = {mm128_load(&mask_b->w64[0]), mm128_load(&mask_b->w64[2])};
+  word128 x2m[2] = {mm128_load(&mask_c->w64[0]), mm128_load(&mask_c->w64[2])};
+  mm128_and_256(x0m, inm, x0m);
+  mm128_and_256(x1m, inm, x1m);
+  mm128_and_256(x2m, inm, x2m);
 
   mm128_shift_left_256(x0m, x0m, 2);
   mm128_shift_left_256(x1m, x1m, 1);
 
-  word128 t0[2] ATTR_ALIGNED(alignof(word128)), t1[2] ATTR_ALIGNED(alignof(word128)),
-      t2[2] ATTR_ALIGNED(alignof(word128));
+  word128 t0[2], t1[2], t2[2];
   mm128_and_256(t0, x1m, x2m);
   mm128_and_256(t1, x0m, x2m);
   mm128_and_256(t2, x0m, x1m);
@@ -240,84 +241,107 @@ static inline void sbox_s128_full(mzd_local_t* in, const word128* mask_a, const
   mm128_shift_right_256(t1, t1, 1);
 
   mm128_xor_256(t0, t0, t1);
-  mm128_xor_256(in->w128, t0, t2);
+  mm128_xor_256(inm, t0, t2);
+  mm128_store(&in->w64[0], inm[0]);
+  mm128_store(&in->w64[2], inm[1]);
 }
 
 #if defined(WITH_LOWMC_129_129_4)
 ATTR_TARGET_S128
 static inline void sbox_s128_lowmc_129_129_4(mzd_local_t* in) {
-  sbox_s128_full(in, mask_129_129_43_a->w128, mask_129_129_43_b->w128, mask_129_129_43_c->w128);
+  sbox_s128_full(in, mask_129_129_43_a, mask_129_129_43_b, mask_129_129_43_c);
 }
 #endif
 
 #if defined(WITH_LOWMC_192_192_4)
 ATTR_TARGET_S128
 static inline void sbox_s128_lowmc_192_192_4(mzd_local_t* in) {
-  sbox_s128_full(in, mask_192_192_64_a->w128, mask_192_192_64_b->w128, mask_192_192_64_c->w128);
+  sbox_s128_full(in, mask_192_192_64_a, mask_192_192_64_b, mask_192_192_64_c);
 }
 #endif
 
 #if defined(WITH_LOWMC_255_255_4)
 ATTR_TARGET_S128
 static inline void sbox_s128_lowmc_255_255_4(mzd_local_t* in) {
-  sbox_s128_full(in, mask_255_255_85_a->w128, mask_255_255_85_b->w128, mask_255_255_85_c->w128);
+  sbox_s128_full(in, mask_255_255_85_a, mask_255_255_85_b, mask_255_255_85_c);
 }
 #endif
 #endif
 
 #if defined(WITH_AVX2)
 ATTR_TARGET_AVX2
-static inline word256 sbox_s256_lowmc_full(const word256 min, const word256 mask_a,
-                                           const word256 mask_b, const word256 mask_c) {
-  word256 x0m ATTR_ALIGNED(alignof(word256)) = mm256_and(min, mask_a);
-  word256 x1m ATTR_ALIGNED(alignof(word256)) = mm256_and(min, mask_b);
-  word256 x2m ATTR_ALIGNED(alignof(word256)) = mm256_and(min, mask_c);
+static inline void sbox_s256_lowmc_full(mzd_local_t* in, const word256 mask_a, const word256 mask_b,
+                                        const word256 mask_c) {
+  const word256 min = mm256_load(in);
+  // a
+  word256 x0m = mm256_and(min, mask_a);
+  // b
+  word256 x1m = mm256_and(min, mask_b);
+  // c
+  word256 x2m = mm256_and(min, mask_c);
 
   x0m = mm256_rotate_left(x0m, 2);
   x1m = mm256_rotate_left(x1m, 1);
 
-  word256 t0 ATTR_ALIGNED(alignof(word256)) = mm256_and(x1m, x2m);
-  word256 t1 ATTR_ALIGNED(alignof(word256)) = mm256_and(x0m, x2m);
-  word256 t2 ATTR_ALIGNED(alignof(word256)) = mm256_and(x0m, x1m);
+  // b & c
+  word256 t0 = mm256_and(x1m, x2m);
+  // a & c
+  word256 t1 = mm256_and(x0m, x2m);
+  // a & b
+  word256 t2 = mm256_and(x0m, x1m);
 
+  // (b & c) ^ a
   t0 = mm256_xor(t0, x0m);
 
+  // a ^ b
   x0m = mm256_xor(x0m, x1m);
-  t1  = mm256_xor(t1, x0m);
-
-  t2 = mm256_xor(t2, x0m);
-  t2 = mm256_xor(t2, x2m);
+  // a ^ b ^ (a & c)
+  t1 = mm256_xor(t1, x0m);
+  // a ^ b ^ c ^ (a & b)
+  t2 = mm256_xor(mm256_xor(t2, x0m), x2m);
 
   t0 = mm256_rotate_right(t0, 2);
   t1 = mm256_rotate_right(t1, 1);
 
-  return mm256_xor(mm256_xor(t0, t1), t2);
+  mm256_store(in, mm256_xor(mm256_xor(t0, t1), t2));
 }
 
 #if defined(WITH_LOWMC_129_129_4)
 ATTR_TARGET_AVX2
 static inline void sbox_s256_lowmc_129_129_4(mzd_local_t* in) {
-  BLOCK(in, 0)->w256 = sbox_s256_lowmc_full(
-      BLOCK(in, 0)->w256, CONST_BLOCK(mask_129_129_43_a, 0)->w256,
-      CONST_BLOCK(mask_129_129_43_b, 0)->w256, CONST_BLOCK(mask_129_129_43_c, 0)->w256);
+  sbox_s256_lowmc_full(in,
+                       mm256_set_4(MASK_129_129_43_A_0, MASK_129_129_43_A_1, MASK_129_129_43_A_2,
+                                   MASK_129_129_43_A_3),
+                       mm256_set_4(MASK_129_129_43_B_0, MASK_129_129_43_B_1, MASK_129_129_43_B_2,
+                                   MASK_129_129_43_B_3),
+                       mm256_set_4(MASK_129_129_43_C_0, MASK_129_129_43_C_1, MASK_129_129_43_C_2,
+                                   MASK_129_129_43_C_3));
 }
 #endif
 
 #if defined(WITH_LOWMC_192_192_4)
 ATTR_TARGET_AVX2
 static inline void sbox_s256_lowmc_192_192_4(mzd_local_t* in) {
-  BLOCK(in, 0)->w256 = sbox_s256_lowmc_full(
-      BLOCK(in, 0)->w256, CONST_BLOCK(mask_192_192_64_a, 0)->w256,
-      CONST_BLOCK(mask_192_192_64_b, 0)->w256, CONST_BLOCK(mask_192_192_64_c, 0)->w256);
+  sbox_s256_lowmc_full(in,
+                       mm256_set_4(MASK_192_192_64_A_0, MASK_192_192_64_A_1, MASK_192_192_64_A_2,
+                                   MASK_192_192_64_A_3),
+                       mm256_set_4(MASK_192_192_64_B_0, MASK_192_192_64_B_1, MASK_192_192_64_B_2,
+                                   MASK_192_192_64_B_3),
+                       mm256_set_4(MASK_192_192_64_C_0, MASK_192_192_64_C_1, MASK_192_192_64_C_2,
+                                   MASK_192_192_64_C_3));
 }
 #endif
 
 #if defined(WITH_LOWMC_255_255_4)
 ATTR_TARGET_AVX2
 static inline void sbox_s256_lowmc_255_255_4(mzd_local_t* in) {
-  BLOCK(in, 0)->w256 = sbox_s256_lowmc_full(
-      BLOCK(in, 0)->w256, CONST_BLOCK(mask_255_255_85_a, 0)->w256,
-      CONST_BLOCK(mask_255_255_85_b, 0)->w256, CONST_BLOCK(mask_255_255_85_c, 0)->w256);
+  sbox_s256_lowmc_full(in,
+                       mm256_set_4(MASK_255_255_85_A_0, MASK_255_255_85_A_1, MASK_255_255_85_A_2,
+                                   MASK_255_255_85_A_3),
+                       mm256_set_4(MASK_255_255_85_B_0, MASK_255_255_85_B_1, MASK_255_255_85_B_2,
+                                   MASK_255_255_85_B_3),
+                       mm256_set_4(MASK_255_255_85_C_0, MASK_255_255_85_C_1, MASK_255_255_85_C_2,
+                                   MASK_255_255_85_C_3));
 }
 #endif
 #endif /* WITH_AVX2 */
@@ -449,48 +473,56 @@ static void sbox_aux_uint64_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* s
 #define picnic3_aux_sbox_bitsliced_mm128(LOWMC_N, XOR, AND, SHL, SHR, bitmask_a, bitmask_b,        \
                                          bitmask_c)                                                \
   do {                                                                                             \
-    word128 a[2] ATTR_ALIGNED(alignof(word128));                                                   \
-    word128 b[2] ATTR_ALIGNED(alignof(word128));                                                   \
-    word128 c[2] ATTR_ALIGNED(alignof(word128));                                                   \
+    word128 bita[2] = {mm128_load(&bitmask_a->w64[0]), mm128_load(&bitmask_a->w64[2])};            \
+    word128 bitb[2] = {mm128_load(&bitmask_b->w64[0]), mm128_load(&bitmask_b->w64[2])};            \
+    word128 bitc[2] = {mm128_load(&bitmask_c->w64[0]), mm128_load(&bitmask_c->w64[2])};            \
+                                                                                                   \
+    word128 aux[2] = {mm128_load(&statein->w64[0]), mm128_load(&statein->w64[2])};                 \
+                                                                                                   \
+    word128 a[2];                                                                                  \
+    word128 b[2];                                                                                  \
+    word128 c[2];                                                                                  \
     /* a */                                                                                        \
-    AND(a, bitmask_a->w128, statein->w128);                                                        \
+    AND(a, bita, aux);                                                                             \
     /* b */                                                                                        \
-    AND(b, bitmask_b->w128, statein->w128);                                                        \
+    AND(b, bitb, aux);                                                                             \
     /* c */                                                                                        \
-    AND(c, bitmask_c->w128, statein->w128);                                                        \
+    AND(c, bitc, aux);                                                                             \
                                                                                                    \
     SHL(a, a, 2);                                                                                  \
     SHL(b, b, 1);                                                                                  \
-    word128 d[2] ATTR_ALIGNED(alignof(word128));                                                   \
-    word128 e[2] ATTR_ALIGNED(alignof(word128));                                                   \
-    word128 f[2] ATTR_ALIGNED(alignof(word128));                                                   \
+                                                                                                   \
+    aux[0] = mm128_load(&stateout->w64[0]);                                                        \
+    aux[1] = mm128_load(&stateout->w64[2]);                                                        \
+    word128 d[2];                                                                                  \
+    word128 e[2];                                                                                  \
+    word128 f[2];                                                                                  \
     /* a */                                                                                        \
-    AND(d, bitmask_a->w128, stateout->w128);                                                       \
+    AND(d, bita, aux);                                                                             \
     /* b */                                                                                        \
-    AND(e, bitmask_b->w128, stateout->w128);                                                       \
+    AND(e, bitb, aux);                                                                             \
     /* c */                                                                                        \
-    AND(f, bitmask_c->w128, stateout->w128);                                                       \
+    AND(f, bitc, aux);                                                                             \
                                                                                                    \
     SHL(d, d, 2);                                                                                  \
     SHL(e, e, 1);                                                                                  \
                                                                                                    \
-    word128 fresh_output_ab[2] ATTR_ALIGNED(alignof(word128));                                     \
-    word128 fresh_output_bc[2] ATTR_ALIGNED(alignof(word128));                                     \
-    word128 fresh_output_ca[2] ATTR_ALIGNED(alignof(word128));                                     \
+    word128 fresh_output_ab[2];                                                                    \
+    word128 fresh_output_bc[2];                                                                    \
+    word128 fresh_output_ca[2];                                                                    \
     XOR(fresh_output_ab, a, b);                                                                    \
     XOR(fresh_output_ca, e, fresh_output_ab);                                                      \
     XOR(fresh_output_bc, d, a);                                                                    \
     XOR(fresh_output_ab, fresh_output_ab, c);                                                      \
     XOR(fresh_output_ab, fresh_output_ab, f);                                                      \
                                                                                                    \
-    word128 t0[2] ATTR_ALIGNED(alignof(word128));                                                  \
-    word128 t1[2] ATTR_ALIGNED(alignof(word128));                                                  \
-    word128 t2[2] ATTR_ALIGNED(alignof(word128));                                                  \
-    mzd_local_t tmp[1], aux[1];                                                                    \
+    word128 t0[2];                                                                                 \
+    word128 t1[2];                                                                                 \
+    word128 t2[2];                                                                                 \
     SHR(t2, fresh_output_ca, 2);                                                                   \
     SHR(t1, fresh_output_bc, 1);                                                                   \
     XOR(t2, t2, t1);                                                                               \
-    XOR(aux->w128, t2, fresh_output_ab);                                                           \
+    XOR(aux, t2, fresh_output_ab);                                                                 \
                                                                                                    \
     /* a & b */                                                                                    \
     AND(t0, a, b);                                                                                 \
@@ -502,21 +534,21 @@ static void sbox_aux_uint64_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* s
     SHR(t1, t1, 1);                                                                                \
     XOR(t2, t2, t1);                                                                               \
     XOR(t2, t2, t0);                                                                               \
-    XOR(aux->w128, aux->w128, t2);                                                                 \
+    XOR(aux, aux, t2);                                                                             \
                                                                                                    \
     bitstream_t parity_tape     = {{tapes->parity_tapes}, tapes->pos};                             \
     bitstream_t last_party_tape = {{tapes->tape[15]}, tapes->pos};                                 \
                                                                                                    \
     /* calculate aux_bits to fix and_helper */                                                     \
-    mzd_from_bitstream(&parity_tape, tmp, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);       \
-    XOR(aux->w128, aux->w128, tmp->w128);                                                          \
-    mzd_from_bitstream(&last_party_tape, tmp, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);   \
-    XOR(aux->w128, aux->w128, tmp->w128);                                                          \
+    w128_from_bitstream(&parity_tape, t0, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);       \
+    XOR(aux, aux, t0);                                                                             \
+    w128_from_bitstream(&last_party_tape, t1, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);   \
+    XOR(aux, aux, t1);                                                                             \
                                                                                                    \
     last_party_tape.position = tapes->pos;                                                         \
-    mzd_to_bitstream(&last_party_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);     \
+    w128_to_bitstream(&last_party_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);    \
     bitstream_t aux_tape = {{tapes->aux_bits}, tapes->aux_pos};                                    \
-    mzd_to_bitstream(&aux_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);            \
+    w128_to_bitstream(&aux_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);           \
                                                                                                    \
     tapes->aux_pos += LOWMC_N;                                                                     \
   } while (0)
@@ -576,77 +608,63 @@ static void sbox_aux_s128_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* sta
 #define IMPL s256
 
 #if defined(WITH_KKW)
-#define picnic3_aux_sbox_bitsliced_mm256(LOWMC_N, XOR, AND, ROL, ROR, bitmask_a, bitmask_b,        \
-                                         bitmask_c)                                                \
+#define picnic3_aux_sbox_bitsliced_mm256(LOWMC_N, bitmask_a, bitmask_b, bitmask_c)                 \
   do {                                                                                             \
-    word256 a ATTR_ALIGNED(alignof(word256));                                                      \
-    word256 b ATTR_ALIGNED(alignof(word256));                                                      \
-    word256 c ATTR_ALIGNED(alignof(word256));                                                      \
+    const word256 bita = bitmask_a, bitb = bitmask_b, bitc = bitmask_c;                            \
+    word256 aux = mm256_load(statein->w64);                                                        \
     /* a */                                                                                        \
-    a = AND(bitmask_a->w256, statein->w256);                                                       \
+    word256 a = mm256_and(bita, aux);                                                              \
     /* b */                                                                                        \
-    b = AND(bitmask_b->w256, statein->w256);                                                       \
+    word256 b = mm256_and(bitb, aux);                                                              \
     /* c */                                                                                        \
-    c = AND(bitmask_c->w256, statein->w256);                                                       \
+    word256 c = mm256_and(bitc, aux);                                                              \
                                                                                                    \
-    a = ROL(a, 2);                                                                                 \
-    b = ROL(b, 1);                                                                                 \
-    word256 d ATTR_ALIGNED(alignof(word256));                                                      \
-    word256 e ATTR_ALIGNED(alignof(word256));                                                      \
-    word256 f ATTR_ALIGNED(alignof(word256));                                                      \
+    a = mm256_rotate_left(a, 2);                                                                   \
+    b = mm256_rotate_left(b, 1);                                                                   \
+                                                                                                   \
+    aux = mm256_load(stateout->w64);                                                               \
     /* d */                                                                                        \
-    d = AND(bitmask_a->w256, stateout->w256);                                                      \
+    word256 d = mm256_and(bita, aux);                                                              \
     /* e */                                                                                        \
-    e = AND(bitmask_b->w256, stateout->w256);                                                      \
+    word256 e = mm256_and(bitb, aux);                                                              \
     /* f */                                                                                        \
-    f = AND(bitmask_c->w256, stateout->w256);                                                      \
+    word256 f = mm256_and(bitc, aux);                                                              \
                                                                                                    \
-    d = ROL(d, 2);                                                                                 \
-    e = ROL(e, 1);                                                                                 \
+    d = mm256_rotate_left(d, 2);                                                                   \
+    e = mm256_rotate_left(e, 1);                                                                   \
                                                                                                    \
-    word256 fresh_output_ab ATTR_ALIGNED(alignof(word256));                                        \
-    word256 fresh_output_bc ATTR_ALIGNED(alignof(word256));                                        \
-    word256 fresh_output_ca ATTR_ALIGNED(alignof(word256));                                        \
-    fresh_output_ab = XOR(a, b);                                                                   \
-    fresh_output_ca = XOR(e, fresh_output_ab);                                                     \
-    fresh_output_bc = XOR(d, a);                                                                   \
-    fresh_output_ab = XOR(fresh_output_ab, c);                                                     \
-    fresh_output_ab = XOR(fresh_output_ab, f);                                                     \
+    word256 fresh_output_ab = mm256_xor(a, b);                                                     \
+    word256 fresh_output_ca = mm256_xor(e, fresh_output_ab);                                       \
+    word256 fresh_output_bc = mm256_xor(d, a);                                                     \
+    fresh_output_ab         = mm256_xor(fresh_output_ab, mm256_xor(c, f));                         \
                                                                                                    \
-    word256 t0 ATTR_ALIGNED(alignof(word256));                                                     \
-    word256 t1 ATTR_ALIGNED(alignof(word256));                                                     \
-    word256 t2 ATTR_ALIGNED(alignof(word256));                                                     \
-    mzd_local_t tmp[1], aux[1];                                                                    \
-    t2        = ROR(fresh_output_ca, 2);                                                           \
-    t1        = ROR(fresh_output_bc, 1);                                                           \
-    t2        = XOR(t2, t1);                                                                       \
-    aux->w256 = XOR(t2, fresh_output_ab);                                                          \
+    word256 t2 = mm256_rotate_right(fresh_output_ca, 2);                                           \
+    word256 t1 = mm256_rotate_right(fresh_output_bc, 1);                                           \
+    t2         = mm256_xor(t2, t1);                                                                \
+    aux        = mm256_xor(t2, fresh_output_ab);                                                   \
                                                                                                    \
     /* a & b */                                                                                    \
-    t0 = AND(a, b);                                                                                \
+    word256 t0 = mm256_and(a, b);                                                                  \
     /* b & c */                                                                                    \
-    t1 = AND(b, c);                                                                                \
+    t1 = mm256_and(b, c);                                                                          \
     /* c & a */                                                                                    \
-    t2        = AND(c, a);                                                                         \
-    t2        = ROR(t2, 2);                                                                        \
-    t1        = ROR(t1, 1);                                                                        \
-    t2        = XOR(t2, t1);                                                                       \
-    t2        = XOR(t2, t0);                                                                       \
-    aux->w256 = XOR(aux->w256, t2);                                                                \
+    t2  = mm256_and(c, a);                                                                         \
+    t2  = mm256_rotate_right(t2, 2);                                                               \
+    t1  = mm256_rotate_right(t1, 1);                                                               \
+    aux = mm256_xor(mm256_xor(aux, t2), mm256_xor(t0, t1));                                        \
                                                                                                    \
     bitstream_t parity_tape     = {{tapes->parity_tapes}, tapes->pos};                             \
     bitstream_t last_party_tape = {{tapes->tape[15]}, tapes->pos};                                 \
                                                                                                    \
     /* calculate aux_bits to fix and_helper */                                                     \
-    mzd_from_bitstream(&parity_tape, tmp, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);       \
-    aux->w256 = XOR(aux->w256, tmp->w256);                                                         \
-    mzd_from_bitstream(&last_party_tape, tmp, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);   \
-    aux->w256 = XOR(aux->w256, tmp->w256);                                                         \
+    t0  = w256_from_bitstream(&parity_tape, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);     \
+    t1  = w256_from_bitstream(&last_party_tape, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N); \
+    aux = mm256_xor(aux, mm256_xor(t0, t1));                                                       \
                                                                                                    \
     last_party_tape.position = tapes->pos;                                                         \
-    mzd_to_bitstream(&last_party_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);     \
+    w256_to_bitstream(&last_party_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);    \
     bitstream_t aux_tape = {{tapes->aux_bits}, tapes->aux_pos};                                    \
-    mzd_to_bitstream(&aux_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);            \
+    w256_to_bitstream(&aux_tape, aux, (LOWMC_N + 63) / (sizeof(uint64_t) * 8), LOWMC_N);           \
                                                                                                    \
     tapes->aux_pos += LOWMC_N;                                                                     \
   } while (0)
@@ -655,27 +673,39 @@ static void sbox_aux_s128_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* sta
 ATTR_TARGET_AVX2
 static void sbox_aux_s256_lowmc_129_129_4(mzd_local_t* statein, mzd_local_t* stateout,
                                           randomTape_t* tapes) {
-  picnic3_aux_sbox_bitsliced_mm256(LOWMC_129_129_4_N, mm256_xor, mm256_and, mm256_shift_left,
-                                   mm256_shift_right, mask_129_129_43_a, mask_129_129_43_b,
-                                   mask_129_129_43_c);
+  picnic3_aux_sbox_bitsliced_mm256(LOWMC_129_129_4_N,
+                                   mm256_set_4(MASK_129_129_43_A_0, MASK_129_129_43_A_1,
+                                               MASK_129_129_43_A_2, MASK_129_129_43_A_3),
+                                   mm256_set_4(MASK_129_129_43_B_0, MASK_129_129_43_B_1,
+                                               MASK_129_129_43_B_2, MASK_129_129_43_B_3),
+                                   mm256_set_4(MASK_129_129_43_C_0, MASK_129_129_43_C_1,
+                                               MASK_129_129_43_C_2, MASK_129_129_43_C_3));
 }
 #endif
 #if defined(WITH_LOWMC_192_192_4)
 ATTR_TARGET_AVX2
 static void sbox_aux_s256_lowmc_192_192_4(mzd_local_t* statein, mzd_local_t* stateout,
                                           randomTape_t* tapes) {
-  picnic3_aux_sbox_bitsliced_mm256(LOWMC_192_192_4_N, mm256_xor, mm256_and, mm256_rotate_left,
-                                   mm256_rotate_right, mask_192_192_64_a, mask_192_192_64_b,
-                                   mask_192_192_64_c);
+  picnic3_aux_sbox_bitsliced_mm256(LOWMC_192_192_4_N,
+                                   mm256_set_4(MASK_192_192_64_A_0, MASK_192_192_64_A_1,
+                                               MASK_192_192_64_A_2, MASK_192_192_64_A_3),
+                                   mm256_set_4(MASK_192_192_64_B_0, MASK_192_192_64_B_1,
+                                               MASK_192_192_64_B_2, MASK_192_192_64_B_3),
+                                   mm256_set_4(MASK_192_192_64_C_0, MASK_192_192_64_C_1,
+                                               MASK_192_192_64_C_2, MASK_192_192_64_C_3));
 }
 #endif
 #if defined(WITH_LOWMC_255_255_4)
 ATTR_TARGET_AVX2
 static void sbox_aux_s256_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* stateout,
                                           randomTape_t* tapes) {
-  picnic3_aux_sbox_bitsliced_mm256(LOWMC_255_255_4_N, mm256_xor, mm256_and, mm256_rotate_left,
-                                   mm256_rotate_right, mask_255_255_85_a, mask_255_255_85_b,
-                                   mask_255_255_85_c);
+  picnic3_aux_sbox_bitsliced_mm256(LOWMC_255_255_4_N,
+                                   mm256_set_4(MASK_255_255_85_A_0, MASK_255_255_85_A_1,
+                                               MASK_255_255_85_A_2, MASK_255_255_85_A_3),
+                                   mm256_set_4(MASK_255_255_85_B_0, MASK_255_255_85_B_1,
+                                               MASK_255_255_85_B_2, MASK_255_255_85_B_3),
+                                   mm256_set_4(MASK_255_255_85_C_0, MASK_255_255_85_C_1,
+                                               MASK_255_255_85_C_2, MASK_255_255_85_C_3));
 }
 #endif
 #endif
@@ -700,8 +730,8 @@ static void sbox_aux_s256_lowmc_255_255_4(mzd_local_t* statein, mzd_local_t* sta
 #endif
 #endif
 
-void lowmc(const lowmc_parameters_t* lowmc, const lowmc_key_t* key, const mzd_local_t* x,
-           mzd_local_t* y) {
+void lowmc_compute(const lowmc_parameters_t* lowmc, const lowmc_key_t* key, const mzd_local_t* x,
+                   mzd_local_t* y) {
   const uint32_t lowmc_id = LOWMC_GET_ID(lowmc);
 #if defined(WITH_OPT)
 #if defined(WITH_AVX2)
@@ -950,8 +980,8 @@ lowmc_implementation_f lowmc_get_implementation(const lowmc_parameters_t* lowmc)
 #endif
 
 #if defined(WITH_ZKBPP)
-void lowmc_store(const lowmc_parameters_t* lowmc, const lowmc_key_t* key, const mzd_local_t* x,
-                 recorded_state_t* state) {
+void lowmc_record_state(const lowmc_parameters_t* lowmc, const lowmc_key_t* key,
+                        const mzd_local_t* x, recorded_state_t* state) {
   const uint32_t lowmc_id = LOWMC_GET_ID(lowmc);
 #if defined(WITH_OPT)
 #if defined(WITH_AVX2)
@@ -1088,17 +1118,17 @@ void lowmc_compute_aux(const lowmc_parameters_t* lowmc, lowmc_key_t* key, random
       /* Instances with full Sbox layer */
 #if defined(WITH_LOWMC_129_129_4)
     case LOWMC_ID(129, 43):
-      lowmc_compute_aux_s256_lowmc_129_129_4(key, tapes);
+      lowmc_aux_s256_lowmc_129_129_4(key, tapes);
       return;
 #endif
 #if defined(WITH_LOWMC_192_192_4)
     case LOWMC_ID(192, 64):
-      lowmc_compute_aux_s256_lowmc_192_192_4(key, tapes);
+      lowmc_aux_s256_lowmc_192_192_4(key, tapes);
       return;
 #endif
 #if defined(WITH_LOWMC_255_255_4)
     case LOWMC_ID(255, 85):
-      lowmc_compute_aux_s256_lowmc_255_255_4(key, tapes);
+      lowmc_aux_s256_lowmc_255_255_4(key, tapes);
       return;
 #endif
     }
@@ -1112,17 +1142,17 @@ void lowmc_compute_aux(const lowmc_parameters_t* lowmc, lowmc_key_t* key, random
       /* Instances with full Sbox layer */
 #if defined(WITH_LOWMC_129_129_4)
     case LOWMC_ID(129, 43):
-      lowmc_compute_aux_s128_lowmc_129_129_4(key, tapes);
+      lowmc_aux_s128_lowmc_129_129_4(key, tapes);
       return;
 #endif
 #if defined(WITH_LOWMC_192_192_4)
     case LOWMC_ID(192, 64):
-      lowmc_compute_aux_s128_lowmc_192_192_4(key, tapes);
+      lowmc_aux_s128_lowmc_192_192_4(key, tapes);
       return;
 #endif
 #if defined(WITH_LOWMC_255_255_4)
     case LOWMC_ID(255, 85):
-      lowmc_compute_aux_s128_lowmc_255_255_4(key, tapes);
+      lowmc_aux_s128_lowmc_255_255_4(key, tapes);
       return;
 #endif
     }
@@ -1136,17 +1166,17 @@ void lowmc_compute_aux(const lowmc_parameters_t* lowmc, lowmc_key_t* key, random
     /* Instances with full Sbox layer */
 #if defined(WITH_LOWMC_129_129_4)
   case LOWMC_ID(129, 43):
-    lowmc_compute_aux_uint64_lowmc_129_129_4(key, tapes);
+    lowmc_aux_uint64_lowmc_129_129_4(key, tapes);
     return;
 #endif
 #if defined(WITH_LOWMC_192_192_4)
   case LOWMC_ID(192, 64):
-    lowmc_compute_aux_uint64_lowmc_192_192_4(key, tapes);
+    lowmc_aux_uint64_lowmc_192_192_4(key, tapes);
     return;
 #endif
 #if defined(WITH_LOWMC_255_255_4)
   case LOWMC_ID(255, 85):
-    lowmc_compute_aux_uint64_lowmc_255_255_4(key, tapes);
+    lowmc_aux_uint64_lowmc_255_255_4(key, tapes);
     return;
 #endif
   }
